@@ -415,35 +415,37 @@ class SoundFile(object):
             raise AttributeError("SoundFile has no attribute %s" % name)
 
     def __len__(self):
-        # strangely, the only way to see the length of a file seems to
-        # be to seek to the end. Returns the number of frames in the
-        # file.
-        curr = self.seek(0)
-        length = self.seek_absolute(-1)
-        self.seek_absolute(curr)
-        return(length)
+        return(self.frames)
+
+    def _get_slice_bounds(self, frame):
+        # get start and stop index from slice, asserting step==1
+        if not isinstance(frame, slice):
+            frame = slice(frame, frame + 1)
+        start, stop, step = frame.indices(len(self))
+        if step != 1:
+            raise RuntimeError("Step size must be 1!")
+        if start > stop:
+            stop = start
+        return start, stop
 
     def __getitem__(self, frame):
-        # access the file as if it where a one-dimensional Numpy
-        # array. Data must be in the form (frames x channels).
-        # Both open slice bounds and negative values are allowed.
-        if not isinstance(frame, slice):
-            frame = slice(frame, frame+1)
-        if frame.start is None:
-            frame = slice(0, frame.stop)
-        if frame.stop is None:
-            frame = slice(frame.start, len(self))
-        if frame.start < 0:
-            frame = slice(len(self)+frame.start, frame.stop)
-        if frame.stop < 0:
-            frame = slice(frame.start, len(self)+frame.stop)
-        if frame.start > frame.stop:
-            frame = slice(frame.stop, frame.start)
+        # access the file as if it where a Numpy array. The data is
+        # returned as numpy array.
+        second_frame = None
+        if isinstance(frame, tuple):
+            if len(frame) > 2:
+                raise AttributeError(
+                    "SoundFile can only be accessed in one or two dimensions")
+            frame, second_frame = frame
+        start, stop = self._get_slice_bounds(frame)
         curr = self.seek(0)
-        self.seek_absolute(frame.start)
-        data = self.read(frame.stop-frame.start)
-        self.seek(curr)
-        return data
+        self.seek_absolute(start)
+        data = self.read(stop - start)
+        self.seek_absolute(curr)
+        if second_frame:
+            return data[(slice(None), second_frame)]
+        else:
+            return data
 
     def __setitem__(self, frame, data):
         # access the file as if it where a one-dimensional Numpy
@@ -451,26 +453,21 @@ class SoundFile(object):
         # Both open slice bounds and negative values are allowed.
         if self._file_mode == read_mode:
             raise RuntimeError("Can not write to read-only file")
-        if not isinstance(frame, slice):
-            frame = slice(frame, frame+1)
-        if frame.start is None:
-            frame = slice(0, frame.stop)
-        if frame.stop is None:
-            frame = slice(frame.start, len(self))
-        if frame.start < 0:
-            frame = slice(len(self)+frame.start, frame.stop)
-        if frame.stop < 0:
-            frame = slice(frame.start, len(self)+frame.stop)
-        if frame.stop-frame.start != len(data):
+        start, stop = self._get_slice_bounds(frame)
+        if stop - start != len(data):
             raise IndexError(
                 "Could not fit data of length %i into slice of length %i" %
-                (len(data), frame.stop-frame.start))
+                (len(data), stop - start))
         curr = self.seek(0)
-        self.seek_absolute(frame.start)
+        self.seek_absolute(start)
         self.write(data)
-        self.seek(curr)
+        self.seek_absolute(curr)
         return data
 
+    def flush(self):
+        """Write unwritten data to disk."""
+        _snd.sf_write_sync(self._file)
+
     def seek(self, frames):
         """Set the read position relative to the current position.
 
@@ -478,7 +475,7 @@ class SoundFile(object):
 
         Returns the new absolute read position in frames.
         """
-        return _snd.sf_seek(self._file, frames, 1)
+        return _snd.sf_seek(self._file, frames, os.SEEK_CUR)
 
     def seek_absolute(self, frames):
         """Set an absolute read position.
@@ -490,16 +487,17 @@ class SoundFile(object):
         Returns the new absolute read position in frames.
         """
         if frames >= 0:
-            return _snd.sf_seek(self._file, frames, 0)
+            return _snd.sf_seek(self._file, frames, os.SEEK_SET)
         else:
-            return _snd.sf_seek(self._file, frames, 2)
+            return _snd.sf_seek(self._file, frames, os.SEEK_END)
 
-    def read(self, frames, format=np.float32):
+    def read(self, frames=-1, format=np.float32):
         """Read a number of frames from the file.
 
         Reads the given number of frames in the given data format from
         the current read position. This also advances the read
         position by the same number of frames.
+        Use frames=-1 to read until the end of the file.
 
         Returns the read data as a (frames x channels) NumPy array.
 
@@ -521,6 +519,9 @@ class SoundFile(object):
         }
         if format not in formats:
             raise ValueError("Can only read int16, int32, float32 and float64")
+        if frames == -1:
+            curr = self.seek(0)
+            frames = self.frames - curr
         data = ffi.new(formats[format], frames*self.channels)
         read = readers[format](self._file, data, frames)
         self._handle_error()
